%{
/*
 * NAME
 *   lexer_l.l -- bogofilter's lexical analyzer
 *
*/
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "bogofilter.h"

// Our lexical analysis is different from Paul Graham's rules: 
//
// We throw away headers that are readily identifiable as dates.
// We throw away text lines that look like BASE64 encoding.
// We throw away all digit strings that don't look like IP address parts.
// We thow away lines beginning with <tab>id<space> -- mailer UDs.
// We throw away various bits of MIME cruft (things that look like
// declarations and boundaries for multipart declarations).
// We throw away *all* tokens of length 1 or 2.
//
// These are optimizations to keep the token lists from bloating.
// The big win is regognizing machine-generated unique IDs that
// we'll never see again and shouldn't 
//
// We don't treat dot between two alphanumerics as a separator,
// because we want to keep domain names and IP addresses together as 
// recognizable units. 
//
// Having done the above, there isn't much need to recognize URLs.  
// If a URL is a spam indicator, very likely any other URL from the
// same site is as well, so the hostname part should be an adequate
// statistical trigger.  
//
// The list of HTML keywords and attributes to be ignored is from the 4.0 DTD.

%}

%option full align nounput noyywrap noreject 8bit
%option outfile="lexer_text_plain.cc"
%option prefix="TextPlain"

BASE64          [A-Za-z0-9/+]{72,}$
IPADDR		[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+

%%

{BASE64}                                        ;
{IPADDR}					{return(TOKEN);}
[A-Za-z$][A-Za-z0-9$'.-]+[A-Za-z0-9$]		{return(TOKEN);}
.						;
\n						;

%%

int get_token_text(FlexLexer *lexer)
{
    int classc;
    while ((classc = lexer->yylex()) > 0)
	if (lexer->YYLeng() <= MAXWORDLEN)
	    break;

    return(classc);
}

#ifdef MAIN
int main(void)
{
    int	t;

    while ((t = get_token()) > 0)
    {
	(void) printf("get_token: %d '%s'\n", t, yytext);
    }
    return 0;
}
#endif /* MAIN */

// The following sets edit modes for GNU EMACS
// Local Variables:
// mode:fundamental
// End:
