/*****************************************************************************

NAME:
   main.c -- detect spam and bogons presented on standard input.

AUTHOR:
   Eric S. Raymond <esr@thyrsus.com>

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <gmime/gmime.h>

#include "bogofilter.h"

//#define MSG_TYPE_PLAIN
#define MSG_TYPE_EMAIL

#ifdef MSG_TYPE_EMAIL
#define BOGODIR		"/.bogofilter/new_db/"
#else
#ifdef MSG_TYPE_PLAIN
#define BOGODIR		"/.bogofilter/news/"
#endif
#endif

int verbose, passthrough;

int main(int argc, char **argv)
{
    int	ch, dump = 0;
    bool reg=false; // register
    const char *class_name_to_add=NULL;
    const char *class_name_to_rem=NULL;
    char directory[PATH_MAX];
    struct stat sb;
    int readerror=0;

    g_mime_init(0);

    strcpy(directory, getenv("HOME"));
    strcat(directory, BOGODIR);

    while ((ch = getopt(argc, argv, "dshSHvplc:")) != EOF) {
      switch(ch) {
      case 'd':
        strcpy(directory, optarg);
        if (directory[strlen(directory)+1] != '/')
          strcat(directory, "/" );
        break;
        
      case 's':
        reg=true;
        class_name_to_add = "spam";
        break;
        
      case 'h':
        reg=true;
        class_name_to_add = "ham";
        break;
        
      case 'S':
        reg=true;
        class_name_to_rem = "spam";
        break;
        
      case 'H':
        reg=true;
        class_name_to_rem = "ham";
        break;
        
      case 'v':
        verbose++;
        break;
        
      case 'p':
        passthrough++;
        break;
        
      case 'l':
        dump = 1;
        break;

      case 'c':
        {
          char *cur_pos=optarg;
          char *delim=NULL;
          while(cur_pos != NULL) {
            delim=strchr(cur_pos,',');
            if(delim!=NULL)
              *delim='\0';

            if(*cur_pos == '-') {
              reg = true;
              class_name_to_rem = strdup(cur_pos+1);
            } else if(*cur_pos == '+') {
              reg = true;
              class_name_to_add = strdup(cur_pos+1);
            } else {
              if(strlen(cur_pos) > 0) {
                reg = true;
                class_name_to_add = strdup(cur_pos);
              }
            }

            if(delim!=NULL)
              cur_pos = delim+1;
            else
              cur_pos = delim;
          }
        }
        break;
      }
    }

    //if(class_name_to_add && class_name_to_add) {
    //  fprintf(stderr,"Can't register a message as spam and ham simultaneously\n");
    //  exit(2);
    //}

    //if(unregister_ham && unregister_spam) {
    //  fprintf(stderr,"Can't unregister a message as spam and ham simultaneously\n");
    //  exit(2);
    //}

    ch = stat(directory, &sb);
    if (!((ch == 0 && S_ISDIR(sb.st_mode))
	  || (errno==ENOENT && mkdir(directory, S_IRUSR|S_IWUSR|S_IXUSR)==0))) {
      fprintf(stderr,"bogofilter: something is wrong with %s.\n", directory);
      exit(2);
    }

    readerror = read_class_list(directory);

    /* readerror is ok, but only if using register_* modes */
    if (readerror!=0 && (class_name_to_add!=NULL && reg)) {
      fprintf(stderr, "Error: can't open list file(s).  readerror=%d\n",readerror);
      exit(2);
    }

    if (dump) {
      if(class_name_to_add) {
        int error;
        error = bogodump(class_name_to_add);
        if(error > 0)
          printf("bogodump error\n");
      }
    } else if(reg) {
      register_words(0,class_name_to_add,class_name_to_rem,directory);
    } else {
      int status=2;
      int class_num=-1;

      if(optind >= argc) {
        status = bogofilter(0,class_num);
        printf("%d\n",class_num);
      } else {
        for(int opt_idx=optind; opt_idx<argc; opt_idx++) {
          FILE *test_file;

          test_file = fopen(argv[opt_idx],"r");
          if(test_file==NULL) {
            printf("problem opening file '%s' for reading\n",argv[opt_idx]);
            continue;
          }

          class_num = -1;
          status = bogofilter(fileno(test_file),class_num);
          printf("%s %d\n",argv[opt_idx],class_num);
          fclose(test_file);
        }
      }
      
    //  if (passthrough) {
    //    int inheaders = 1;
    //    
    //    for (textend=&textblocks; textend->block; textend=textend->next) {
    //      if (inheaders && strncmp(textend->block, "Subject:", 8)==0)
    //        printf("X-Spam-Status: %s, tests=bogofilter\n", 
    //               status ? "No" : "Yes");
    //      else if (strcmp(textend->block, "\n") == 0 && inheaders == 0)
    //        inheaders = 0;
    //      (void) fputs(textend->block, stdout);
    //    }
    //  }
      exit(status);
    }

    exit(0);
}

// End
